@@ -30,7 +30,7 @@
 
 #define __GNOME_FONT_FACE_C__
 
-#include <config.h>
+#include "config.h"
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
@@ -43,7 +43,7 @@
 #include <locale.h>
 #include <libgnomeprint/gnome-font-private.h>
 #include "gnome-print-i18n.h"
-#include "gp-fontmap.h"
+#include "gnome-fontmap.h"
 #include "gp-truetype-utils.h"
 #include "gp-tt-t1.h"
 #include "gnome-font-family.h"
@@ -76,11 +76,8 @@ static void gnome_font_face_get_prop (GObject *o, guint id, GValue *value, GPara
 static void gff_load_metrics (GnomeFontFace *face, gint glyph);
 static void gff_load_outline (GnomeFontFace *face, gint glyph);
 
-GFPSObject *gff_pso_from_type1 (const GnomeFontFace * face, GPFontEntry *entry);
-GFPSObject *gff_pso_from_truetype (const GnomeFontFace *face, GPFontEntry *entry);
-GFPSObject *gff_empty_pso (const GnomeFontFace *face, GPFontEntry *entry);
-static void gf_pso_sprintf (GFPSObject * pso, const gchar * format, ...);
-static void gf_pso_ensure_space (GFPSObject * pso, gint size);
+static void gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...);
+static void gf_pso_ensure_space (GnomeFontPsObject * pso, gint size);
 
 #define GFE_IS_T1_A(e) (((e)->type == GP_FONT_ENTRY_TYPE1) || ((e)->type == GP_FONT_ENTRY_TYPE1_ALIAS))
 #define GFE_IS_TT(e) ((e)->type == GP_FONT_ENTRY_TRUETYPE)
@@ -177,7 +174,8 @@ gnome_font_face_finalize (GObject * object)
 	if (face->glyphs) {
 		gint i;
 		for (i = 0; i < face->num_glyphs; i++) {
-			if (face->glyphs[i].bpath) g_free (face->glyphs[i].bpath);
+			if (face->glyphs[i].bpath)
+				g_free (face->glyphs[i].bpath);
 		}
 		g_free (face->glyphs);
 		face->glyphs = NULL;
@@ -358,9 +356,11 @@ gnome_font_face_get_glyph_stdadvance (GnomeFontFace * face, gint glyph, ArtPoint
 		return NULL;
 	}
 
-	if ((glyph < 0) || (glyph >= face->num_glyphs)) glyph = 0;
+	if ((glyph < 0) || (glyph >= face->num_glyphs))
+		glyph = 0;
 
-	if (!face->glyphs[glyph].metrics) gff_load_metrics ((GnomeFontFace *) face, glyph);
+	if (!face->glyphs[glyph].metrics)
+		gff_load_metrics ((GnomeFontFace *) face, glyph);
 
 	*advance = face->glyphs[glyph].advance;
 
@@ -379,9 +379,11 @@ gnome_font_face_get_glyph_stdbbox (GnomeFontFace * face, gint glyph, ArtDRect *
 		return NULL;
 	}
 
-	if ((glyph < 0) || (glyph >= face->num_glyphs)) glyph = 0;
+	if ((glyph < 0) || (glyph >= face->num_glyphs))
+		glyph = 0;
 
-	if (!face->glyphs[glyph].metrics) gff_load_metrics ((GnomeFontFace *) face, glyph);
+	if (!face->glyphs[glyph].metrics)
+		gff_load_metrics ((GnomeFontFace *) face, glyph);
 
 	*bbox = face->glyphs[glyph].bbox;
 
@@ -399,9 +401,11 @@ gnome_font_face_get_glyph_stdoutline (GnomeFontFace * face, gint glyph)
 		return NULL;
 	}
 
-	if ((glyph < 0) || (glyph >= face->num_glyphs)) glyph = 0;
+	if ((glyph < 0) || (glyph >= face->num_glyphs))
+		glyph = 0;
 
-	if (!face->glyphs[glyph].bpath) gff_load_outline ((GnomeFontFace *) face, glyph);
+	if (!face->glyphs[glyph].bpath)
+		gff_load_outline ((GnomeFontFace *) face, glyph);
 
 	return face->glyphs[glyph].bpath;
 }
@@ -418,8 +422,10 @@ gnome_font_face_get_glyph_stdkerning (GnomeFontFace *face, gint glyph0, gint gly
 		return NULL;
 	}
 
-	if ((glyph0 < 0) || (glyph0 >= face->num_glyphs)) glyph0 = 0;
-	if ((glyph1 < 0) || (glyph1 >= face->num_glyphs)) glyph1 = 0;
+	if ((glyph0 < 0) || (glyph0 >= face->num_glyphs))
+		glyph0 = 0;
+	if ((glyph1 < 0) || (glyph1 >= face->num_glyphs))
+		glyph1 = 0;
 
 	if (!FT_HAS_KERNING (face->ft_face)) {
 		kerning->x = kerning->y = 0.0;
@@ -506,7 +512,8 @@ gnome_font_face_find_closest (const guchar *name)
 
 	map = gp_fontmap_get ();
 
-	if (name) face = gnome_font_face_find (name);
+	if (name)
+		face = gnome_font_face_find (name);
 
 	if (!face) {
 		/* No match, try default */
@@ -520,9 +527,11 @@ gnome_font_face_find_closest (const guchar *name)
 		c[127] = '\0';
 		p = c;
 		while (isalpha (*p) || (*p == '_')) p += 1;
-		if (*p) *p = '\0';
+		if (*p)
+			*p = '\0';
 		e = g_hash_table_lookup (map->defaultsdict, c);
-		if (!e) e = g_hash_table_lookup (map->defaultsdict, "C");
+		if (!e)
+			e = g_hash_table_lookup (map->defaultsdict, "C");
 		if (e) {
 			if (e->face) {
 				gnome_font_face_ref (e->face);
@@ -577,7 +586,8 @@ gnome_font_face_find_closest_from_weight_slant (const guchar *family, GnomeFontW
 	for (l = map->fonts; l != NULL; l = l->next) {
 		entry = (GPFontEntry *) l->data;
 		if (!strcasecmp (family, entry->familyname)) {
-			if (entry->type == GP_FONT_ENTRY_ALIAS) entry = ((GPFontEntryAlias *) entry)->ref;
+			if (entry->type == GP_FONT_ENTRY_ALIAS)
+				entry = ((GPFontEntryAlias *) entry)->ref;
 			dist = abs (weight - entry->Weight) +
 				100 * (italic != (entry->ItalicAngle != 0));
 			if (dist < best_dist) {
@@ -587,12 +597,14 @@ gnome_font_face_find_closest_from_weight_slant (const guchar *family, GnomeFontW
 		}
 	}
 
-	if (best) face = gnome_font_face_find (best->name);
+	if (best)
+		face = gnome_font_face_find (best->name);
 
 	gp_fontmap_release (map);
 
 	/* If nothing found, go with default */
-	if (!face) return gnome_font_face_find (NULL);
+	if (!face)
+		return gnome_font_face_find (NULL);
 
 	return face;
 }
@@ -608,10 +620,12 @@ gnome_font_face_find_from_family_and_style (const guchar *family, const guchar *
 	g_return_val_if_fail (style != NULL, NULL);
 
 	gff = gnome_font_family_new (family);
-	if (!gff) return gnome_font_face_find (NULL);
+	if (!gff)
+		return gnome_font_face_find (NULL);
 	face = gnome_font_family_get_face_by_stylename (gff, style);
 	gnome_font_family_unref (gff);
-	if (!face) return gnome_font_face_find (NULL);
+	if (!face)
+		return gnome_font_face_find (NULL);
 
 	return face;
 }
@@ -732,7 +746,8 @@ gnome_font_face_lookup_default (GnomeFontFace * face, gint unicode)
 	}
 
 	/* fixme: Nobody should ask mapping of 0 */
-	if (unicode < 1) return 0;
+	if (unicode < 1)
+		return 0;
 
 	return FT_Get_Char_Index (face->ft_face, unicode);
 }
@@ -805,7 +820,7 @@ gff_load (GnomeFontFace *face)
 
 	face->ft2ps = 1000.0 / ft_face->units_per_EM;
 
-	face->bbox.x0 = ft_face->bbox.xMin * face->ft2ps;
+	face->bbox.x0 = ft_face->bbox.xMin / face->ft2ps;
 	face->bbox.y0 = ft_face->bbox.yMin * face->ft2ps;
 	face->bbox.x1 = ft_face->bbox.xMax * face->ft2ps;
 	face->bbox.y1 = ft_face->bbox.yMax * face->ft2ps;
@@ -823,9 +838,7 @@ gff_load_metrics (GnomeFontFace *face, gint glyph)
 
 	gi = face->glyphs + glyph;
 
-	FT_Load_Glyph (face->ft_face, glyph,
-		       FT_LOAD_NO_SCALE | FT_LOAD_NO_HINTING |
-		       FT_LOAD_NO_BITMAP | FT_LOAD_IGNORE_TRANSFORM);
+	FT_Load_Glyph (face->ft_face, glyph, FT_LOAD_NO_SCALE | FT_LOAD_NO_HINTING | FT_LOAD_NO_BITMAP);
 
         gi->bbox.x0 = -face->ft_face->glyph->metrics.horiBearingX * face->ft2ps;
 	gi->bbox.y1 = face->ft_face->glyph->metrics.horiBearingY * face->ft2ps;
@@ -986,29 +999,30 @@ gff_ol2bp (FT_Outline * ol, gdouble transform[])
 
 /* PSO (PostScriptObject stuff */
 
-static void gff_pso_ensure_buffer_t1 (GFPSObject *pso);
-static void gff_pso_ensure_buffer_tt (GFPSObject *pso);
-static void gff_pso_ensure_buffer_empty (GFPSObject *pso);
-static void gf_pso_sprintf (GFPSObject * pso, const gchar * format, ...);
-static void gf_pso_ensure_space (GFPSObject *pso, gint size);
+static void gff_pso_ensure_buffer_t1 (GnomeFontPsObject *pso);
+static void gff_pso_ensure_buffer_tt (GnomeFontPsObject *pso);
+static void gff_pso_ensure_buffer_empty (GnomeFontPsObject *pso);
+static void gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...);
+static void gf_pso_ensure_space (GnomeFontPsObject *pso, gint size);
 
 /*
  * Generates intial PSObject
  */
 
-GFPSObject *
+GnomeFontPsObject *
 gnome_font_face_pso_new (GnomeFontFace *face, guchar *residentname)
 {
-	GFPSObject *pso;
+	GnomeFontPsObject *pso;
 
 	g_return_val_if_fail (face != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
 
-	pso = g_new0 (GFPSObject, 1);
+	pso = g_new0 (GnomeFontPsObject, 1);
 
 	pso->face = face;
 	g_object_ref (G_OBJECT (face));
-	if (residentname) pso->residentname = g_strdup (residentname);
+	if (residentname)
+		pso->residentname = g_strdup (residentname);
 	pso->encodedname = g_strdup_printf ("GnomeUni-%s", face->entry->psname);
 	pso->bufsize = 0;
 	pso->length = 0;
@@ -1031,11 +1045,12 @@ gnome_font_face_pso_new (GnomeFontFace *face, guchar *residentname)
 #define PSO_GLYPH_MARKED(pso,g) (pso->glyphs[(g) >> 5] & (1 << ((g) & 0x1f)))
 
 void
-gnome_font_face_pso_mark_glyph (GFPSObject *pso, gint glyph)
+gnome_font_face_pso_mark_glyph (GnomeFontPsObject *pso, gint glyph)
 {
 	g_return_if_fail (pso != NULL);
 
-	if (!pso->glyphs) return;
+	if (!pso->glyphs)
+		return;
 
 	glyph = CLAMP (glyph, 0, pso->num_glyphs);
 
@@ -1043,20 +1058,26 @@ gnome_font_face_pso_mark_glyph (GFPSObject *pso, gint glyph)
 }
 
 void
-gnome_font_face_pso_free (GFPSObject *pso)
+gnome_font_face_pso_free (GnomeFontPsObject *pso)
 {
 	g_return_if_fail (pso != NULL);
 
-	if (pso->face) g_object_unref (G_OBJECT (pso->face));
-	if (pso->residentname) g_free (pso->residentname);
-	if (pso->encodedname) g_free (pso->encodedname);
-	if (pso->glyphs) g_free (pso->glyphs);
-	if (pso->buf) g_free (pso->buf);
+	if (pso->face)
+		g_object_unref (G_OBJECT (pso->face));
+	if (pso->residentname)
+		g_free (pso->residentname);
+	if (pso->encodedname)
+		g_free (pso->encodedname);
+	if (pso->glyphs)
+		g_free (pso->glyphs);
+	if (pso->buf)
+		g_free (pso->buf);
+	
 	g_free (pso);
 }
 
 void
-gnome_font_face_pso_ensure_buffer (GFPSObject *pso)
+gnome_font_face_pso_ensure_buffer (GnomeFontPsObject *pso)
 {
 	g_return_if_fail (pso != NULL);
 
@@ -1079,7 +1100,7 @@ gnome_font_face_pso_ensure_buffer (GFPSObject *pso)
 #define INT32_LSB(q) ((q)[0] + ((q)[1] << 8) + ((q)[2] << 16) + ((q)[3] << 24))
 
 static void
-gff_pso_ensure_buffer_t1 (GFPSObject *pso)
+gff_pso_ensure_buffer_t1 (GnomeFontPsObject *pso)
 {
 	GPFontEntryT1 *t1;
 	struct stat st;
@@ -1226,7 +1247,8 @@ gff_pso_ensure_buffer_t1 (GFPSObject *pso)
 				gchar c[256];
 				FT_Error status;
 				glyph = 256 * i + j;
-				if (glyph >= nglyphs) glyph = 0;
+				if (glyph >= nglyphs)
+					glyph = 0;
 				status = FT_Get_Glyph_Name (pso->face->ft_face, glyph, c, 256);
 				if (status != FT_Err_Ok) {
 					g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, glyph, t1->pfb.name);
@@ -1257,7 +1279,7 @@ gff_pso_ensure_buffer_t1 (GFPSObject *pso)
  */
 
 static void
-gff_pso_ensure_buffer_tt (GFPSObject *pso)
+gff_pso_ensure_buffer_tt (GnomeFontPsObject *pso)
 {
 	GPFontEntryTT *tt;
 	struct stat st;
@@ -1412,7 +1434,8 @@ gff_pso_ensure_buffer_tt (GFPSObject *pso)
 			for (j = 0; j < 256; j++) {
 				gint glyph;
 				glyph = 256 * i + j;
-				if (glyph >= nglyphs) glyph = 0;
+				if (glyph >= nglyphs)
+					glyph = 0;
 				if ((glyph == 0) || !PSO_GLYPH_MARKED (pso, glyph)) {
 					gf_pso_sprintf (pso, ((j & 0xf) == 0xf) ? "/.notdef\n" : "/.notdef ");
 				} else {
@@ -1436,7 +1459,7 @@ gff_pso_ensure_buffer_tt (GFPSObject *pso)
 }
 
 static void
-gff_pso_ensure_buffer_empty (GFPSObject *pso)
+gff_pso_ensure_buffer_empty (GnomeFontPsObject *pso)
 {
 	pso->length = 0;
 
@@ -1466,7 +1489,7 @@ gff_pso_ensure_buffer_empty (GFPSObject *pso)
 }
 
 static void
-gf_pso_sprintf (GFPSObject * pso, const gchar * format, ...)
+gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...)
 {
 	va_list arguments;
 	gchar *oldlocale;
@@ -1491,7 +1514,7 @@ gf_pso_sprintf (GFPSObject * pso, const gchar * format, ...)
 }
 
 static void
-gf_pso_ensure_space (GFPSObject *pso, gint size)
+gf_pso_ensure_space (GnomeFontPsObject *pso, gint size)
 {
 	if (pso->length + size > pso->bufsize) {
 		if (pso->bufsize < 1) {
@@ -1632,12 +1655,15 @@ gnome_font_face_get_glyph_ps_name (GnomeFontFace *face, gint glyph)
 		return ".notdef";
 	}
 
-	if (!sgd) sgd = g_hash_table_new (g_str_hash, g_str_equal);
+	if (!sgd)
+		sgd = g_hash_table_new (g_str_hash, g_str_equal);
 
-	if ((glyph < 0) || (glyph >= face->num_glyphs)) glyph = 0;
+	if ((glyph < 0) || (glyph >= face->num_glyphs))
+		glyph = 0;
 
 	status = FT_Get_Glyph_Name (face->ft_face, glyph, c, 256);
-	if (status != FT_Err_Ok) return ".notdef";
+	if (status != FT_Err_Ok)
+		return ".notdef";
 
 	name = g_hash_table_lookup (sgd, c);
 	if (!name) {
