@@ -25,8 +25,9 @@
 
 #define __GNOME_FONT_FAMILY_C__
 
+#include "config.h"
 #include <string.h>
-#include "gp-fontmap.h"
+#include "gnome-fontmap.h"
 #include "gnome-font-private.h"
 #include "gnome-font-family.h"
 
