@@ -1,12 +1,6 @@
-#define __GNOME_GLYPHLIST_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Copyright (C) 2000-2001 Ximian Inc. and authors
- *
- *  Authors:
- *    Lauris Kaplinski <lauris@ximian.com>
- *
- *  Experimental device independent rich text representation system
+ *  gnome-glyphlist.c: Device independent rich text representation system
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public License
@@ -21,8 +15,17 @@
  *  You should have received a copy of the GNU Library General Public
  *  License along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *  Authors:
+ *    Lauris Kaplinski <lauris@ximian.com>
+ *
+ *  Copyright (C) 2000-2001 Ximian Inc. and authors
+ *
  */
 
+#define __GNOME_GLYPHLIST_C__
+
+#include "config.h"
 #include <string.h>
 #include "gnome-glyphlist-private.h"
 
