@@ -1,13 +1,31 @@
-#define GP_PATH_C
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * GPPath
+ *  gp-path.c:
  *
- * (C) 1999-2000 Lauris Kaplinski <lauris@ariman.ee>
- * Released under LGPL
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
+ *
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *
+ *  Authors:
+ *    Lauris Kaplinski <lauris@ariman.ee>
+ *
+ *  Copyrithg (C) 1999-2000 authors
  *
  */
 
+#define GP_PATH_C
+
+#include "config.h"
 #include <string.h>
 #include <libart_lgpl/art_misc.h>
 #include "gp-path.h"
@@ -460,7 +478,8 @@ gp_path_close_all (const GPPath * path)
 		}
 	}
 
-	if ((!closed) && ((start->x3 != p->x3) || (start->y3 != p->y3))) {
+	if (!closed && (p->code == ART_END ||
+			start->x3 != p->x3 || (start->y3 != p->y3))) {
 		d->code = ART_LINETO;
 		d->x3 = start->x3;
 		d->y3 = start->y3;
