@@ -1,44 +1,52 @@
-#define __GPA_OPTION_C__
-
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ *  gpa-option.c: 
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU Library General Public License
+ *  as published by the Free Software Foundation; either version 2 of
+ *  the License, or (at your option) any later version.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Library General Public License for more details.
  *
- * Authors :
- *   Chema Celorio <chema@celorio.com>
- *   Lauris Kaplinski <lauris@ximian.com>
+ *  You should have received a copy of the GNU Library General Public
+ *  License along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
+ *  Authors :
+ *    Jose M. Celorio <chema@ximian.com>
+ *    Lauris Kaplinski <lauris@ximian.com>
+ *
+ *  Copyright (C) 2000-2001 Ximian, Inc. and Jose M. Celorio
  *
  */
 
+#include <config.h>
+
 #include <string.h>
-#include <libxml/xmlmemory.h>
+#include <libxml/globals.h>
+
 #include "gpa-utils.h"
 #include "gpa-value.h"
+#include "gpa-key.h"
 #include "gpa-option.h"
+#include "gpa-root.h"
 
 static void gpa_option_class_init (GPAOptionClass *klass);
 static void gpa_option_init (GPAOption *option);
 static void gpa_option_finalize (GObject *object);
 
-static GPANode *gpa_option_duplicate (GPANode *node);
-static gboolean gpa_option_verify (GPANode *node);
-static guchar *gpa_option_get_value (GPANode *node);
-static GPANode *gpa_option_get_child (GPANode *node, GPANode *ref);
-static GPANode *gpa_option_lookup (GPANode *node, const guchar *path);
-static void gpa_option_modified (GPANode *node);
+static GPANode * gpa_option_duplicate (GPANode *node);
+static gboolean  gpa_option_verify    (GPANode *node);
+static guchar *  gpa_option_get_value (GPANode *node);
+static GPANode * gpa_option_get_child (GPANode *node, GPANode *previous_child);
+static GPANode * gpa_option_lookup    (GPANode *node, const guchar *path);
+static void      gpa_option_modified  (GPANode *node, guint flags);
+
+static GPANode *gpa_option_create_key_private (GPAOption *option);
 
 static GPAOption *gpa_option_new_node_from_tree (xmlNodePtr tree, const guchar *id);
 static GPAOption *gpa_option_new_key_from_tree (xmlNodePtr tree, const guchar *id);
@@ -92,11 +100,13 @@ gpa_option_class_init (GPAOptionClass *klass)
 	object_class->finalize = gpa_option_finalize;
 
 	node_class->duplicate = gpa_option_duplicate;
-	node_class->verify = gpa_option_verify;
+	node_class->verify    = gpa_option_verify;
 	node_class->get_value = gpa_option_get_value;
 	node_class->get_child = gpa_option_get_child;
-	node_class->lookup = gpa_option_lookup;
-	node_class->modified = gpa_option_modified;
+	node_class->lookup    = gpa_option_lookup;
+	node_class->modified  = gpa_option_modified;
+
+	klass->create_key = gpa_option_create_key_private;
 }
 
 static void
@@ -104,7 +114,7 @@ gpa_option_init (GPAOption *option)
 {
 	option->type = GPA_OPTION_TYPE_NONE;
 	option->children = NULL;
-	option->value = NULL;
+	option->value    = NULL;
 }
 
 static void
@@ -139,7 +149,7 @@ gpa_option_duplicate (GPANode *node)
 
 	option = GPA_OPTION (node);
 
-	new = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, node->id);
+	new = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, gpa_node_id (node));
 	new->type = option->type;
 	if (option->name) {
 		new->name = gpa_node_attach (GPA_NODE (new), gpa_node_duplicate (option->name));
@@ -152,7 +162,8 @@ gpa_option_duplicate (GPANode *node)
 	for (child = option->children; child != NULL; child = child->next) {
 		GPANode *newchild;
 		newchild = gpa_node_duplicate (child);
-		if (newchild) l = g_slist_prepend (l, newchild);
+		if (newchild)
+			l = g_slist_prepend (l, newchild);
 	}
 
 	while (l) {
@@ -176,61 +187,86 @@ gpa_option_verify (GPANode *node)
 	option = GPA_OPTION (node);
 
 	/* Each option has to have Id */
-	if (!node->id) return FALSE;
+	if (!GPA_NODE_ID_EXISTS (node))
+		return FALSE;
 
 	switch (option->type) {
 	case GPA_OPTION_TYPE_NODE:
 		/* !Name, !Value, children */
-		if (option->name) return FALSE;
-		if (option->value) return FALSE;
-		if (!option->children) return FALSE;
+		if (option->name)
+			return FALSE;
+		if (option->value)
+			return FALSE;
+		if (!option->children)
+			return FALSE;
 		for (child = option->children; child != NULL; child = child->next) {
-			if (!GPA_IS_OPTION (child)) return FALSE;
-			if (!gpa_node_verify (child)) return FALSE;
+			if (!GPA_IS_OPTION (child))
+				return FALSE;
+			if (!gpa_node_verify (child))
+				return FALSE;
 		}
 		break;
 	case GPA_OPTION_TYPE_KEY:
 		/* !Name, Value || children */
-		if (option->name) return FALSE;
-		if (!option->value && !option->children) return FALSE;
+		if (option->name)
+			return FALSE;
+		if (!option->value && !option->children)
+			return FALSE;
 		for (child = option->children; child != NULL; child = child->next) {
-			if (!GPA_IS_OPTION (child)) return FALSE;
-			if (GPA_OPTION (child)->type != GPA_OPTION_TYPE_KEY) return FALSE;
-			if (!gpa_node_verify (child)) return FALSE;
+			if (!GPA_IS_OPTION (child))
+				return FALSE;
+			if (GPA_OPTION (child)->type != GPA_OPTION_TYPE_KEY)
+				return FALSE;
+			if (!gpa_node_verify (child))
+				return FALSE;
 		}
 		break;
 	case GPA_OPTION_TYPE_LIST:
 		/* List should not have name */
-		if (option->name) return FALSE;
+		if (option->name)
+			return FALSE;
 		/* List has to have default value */
-		if (!option->value) return FALSE;
+		if (!option->value)
+			return FALSE;
 		/* List has to have children of type item */
-		if (!option->children) return FALSE;
+		if (!option->children)
+			return FALSE;
 		for (child = option->children; child != NULL; child = child->next) {
-			if (!GPA_IS_OPTION (child)) return FALSE;
-			if (GPA_OPTION (child)->type != GPA_OPTION_TYPE_ITEM) return FALSE;
-			if (!gpa_node_verify (child)) return FALSE;
+			if (!GPA_IS_OPTION (child))
+				return FALSE;
+			if (GPA_OPTION (child)->type != GPA_OPTION_TYPE_ITEM)
+				return FALSE;
+			if (!gpa_node_verify (child))
+				return FALSE;
 		}
 		break;
 	case GPA_OPTION_TYPE_ITEM:
 		/* Item has to have name */
-		if (!option->name) return FALSE;
-		if (!gpa_node_verify (option->name)) return FALSE;
+		if (!option->name)
+			return FALSE;
+		if (!gpa_node_verify (option->name))
+			return FALSE;
 		/* Item should not have value */
-		if (option->value) return FALSE;
+		if (option->value)
+			return FALSE;
 		/* Item may have children */
 		for (child = option->children; child != NULL; child = child->next) {
-			if (!GPA_IS_OPTION (child)) return FALSE;
-			if (!gpa_node_verify (child)) return FALSE;
+			if (!GPA_IS_OPTION (child))
+				return FALSE;
+			if (!gpa_node_verify (child))
+				return FALSE;
 		}
 		break;
 	case GPA_OPTION_TYPE_STRING:
 		/* String should not have name */
-		if (option->name) return FALSE;
+		if (option->name)
+			return FALSE;
 		/* String should have value */
-		if (!option->value) return FALSE;
+		if (!option->value)
+			return FALSE;
 		/* String should not have children */
-		if (option->children) return FALSE;
+		if (option->children)
+			return FALSE;
 		break;
 	default:
 		return FALSE;
@@ -245,27 +281,31 @@ gpa_option_get_value (GPANode *node)
 
 	option = GPA_OPTION (node);
 
-	if (option->value) return g_strdup (option->value);
-	if (node->id) return g_strdup (node->id);
+	if (option->value)
+		return g_strdup (option->value);
+	if (GPA_NODE_ID_EXISTS (node))
+		return g_strdup (gpa_node_id (node));
 
 	return NULL;
 }
 
 static GPANode *
-gpa_option_get_child (GPANode *node, GPANode *ref)
+gpa_option_get_child (GPANode *node, GPANode *previous_child)
 {
 	GPAOption *option;
 
 	option = GPA_OPTION (node);
 
-	if (!ref) {
-		if (option->children) gpa_node_ref (option->children);
-		return option->children;
+	if (previous_child) {
+		if (previous_child->next)
+			gpa_node_ref (previous_child->next);
+		return previous_child->next;
 	}
+	
+	if (option->children)
+		gpa_node_ref (option->children);
 
-	if (ref->next) gpa_node_ref (ref->next);
-
-	return ref->next;
+	return option->children;
 }
 
 static GPANode *
@@ -279,7 +319,8 @@ gpa_option_lookup (GPANode *node, const guchar *path)
 	option = GPA_OPTION (node);
 
 	if (!strncmp (path, "Name", 4)) {
-		if (!option->name) return NULL;
+		if (!option->name)
+			return NULL;
 		if (!path[4]) {
 			gpa_node_ref (GPA_NODE (option->name));
 			return GPA_NODE (option->name);
@@ -299,8 +340,10 @@ gpa_option_lookup (GPANode *node, const guchar *path)
 	}
 
 	for (child = option->children; child != NULL; child = child->next) {
+		const guchar *cid;
 		g_assert (GPA_IS_OPTION (child));
-		if (child->id && strlen (child->id) == len && !strncmp (child->id, path, len)) {
+		cid = gpa_node_id (child);
+		if (cid && strlen (cid) == len && !strncmp (cid, path, len)) {
 			if (!next) {
 				gpa_node_ref (child);
 				return child;
@@ -314,30 +357,106 @@ gpa_option_lookup (GPANode *node, const guchar *path)
 }
 
 static void
-gpa_option_modified (GPANode *node)
+gpa_option_modified (GPANode *node, guint flags)
 {
 	GPAOption *option;
 	GPANode *child;
 
 	option = GPA_OPTION (node);
 
-	if (option->name && (GPA_NODE_FLAGS (option->name) & GPA_MODIFIED_FLAG)) {
-		gpa_node_emit_modified (option->name);
+	if (option->name && (GPA_NODE_FLAGS (option->name) & GPA_NODE_MODIFIED_FLAG)) {
+		gpa_node_emit_modified (option->name, 0);
 	}
 
 	child = option->children;
 	while (child) {
 		GPANode *next;
 		next = child->next;
-		if (GPA_NODE_FLAGS (child) & GPA_MODIFIED_FLAG) {
+		if (GPA_NODE_FLAGS (child) & GPA_NODE_MODIFIED_FLAG) {
 			gpa_node_ref (child);
-			gpa_node_emit_modified (child);
+			gpa_node_emit_modified (child, 0);
 			gpa_node_unref (child);
 		}
 		child = next;
 	}
 }
 
+static GPANode *
+gpa_option_create_key_private (GPAOption *option)
+{
+	GPAKey *key;
+	GPANode *child;
+	GPANode *ich;
+	GSList *l;
+
+	key = NULL;
+
+	switch (option->type) {
+	case GPA_OPTION_TYPE_NODE:
+	case GPA_OPTION_TYPE_KEY:
+	case GPA_OPTION_TYPE_STRING:
+		key = (GPAKey *) gpa_node_new (GPA_TYPE_KEY, GPA_NODE_ID (option));
+		key->option = GPA_NODE (option);
+		gpa_node_ref (key->option);
+		if (option->value)
+			key->value = g_strdup (option->value);
+		l = NULL;
+		for (child = option->children; child != NULL; child = child->next) {
+			GPANode *kch;
+			kch = gpa_option_create_key (GPA_OPTION (child));
+			if (kch) {
+				l = g_slist_prepend (l, kch);
+			}
+		}
+		while (l) {
+			GPANode *kch;
+			kch = GPA_NODE (l->data);
+			l = g_slist_remove (l, kch);
+			kch->parent = GPA_NODE (key);
+			kch->next = key->children;
+			key->children = kch;
+		}
+		break;
+	case GPA_OPTION_TYPE_LIST:
+		key = (GPAKey *) gpa_node_new (GPA_TYPE_KEY, GPA_NODE_ID (option));
+		key->option = GPA_NODE (option);
+		gpa_node_ref (key->option);
+		key->value = g_strdup (option->value);
+		child = (GPANode *) gpa_option_get_child_by_id (option, option->value);
+		if (!child) {
+			g_warning ("Could not get child from option by id: [option:%s, id:%s]\n",
+				   gpa_node_id (GPA_NODE (option)), option->value);
+			break;
+		}
+		l = NULL;
+		for (ich = GPA_OPTION (child)->children; ich != NULL; ich = ich->next) {
+			GPANode *kch;
+			kch = gpa_option_create_key (GPA_OPTION (ich));
+			if (kch)
+				l = g_slist_prepend (l, kch);
+		}
+		while (l) {
+			GPANode *kch;
+			kch = GPA_NODE (l->data);
+			l = g_slist_remove (l, kch);
+			kch->parent = GPA_NODE (key);
+			kch->next = key->children;
+			key->children = kch;
+		}
+		gpa_node_unref (child);
+		break;
+		break;
+	case GPA_OPTION_TYPE_ITEM:
+		/* No key for item */
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+
+	return (GPANode *) key;
+}
+
 GPANode *
 gpa_option_new_from_tree (xmlNodePtr tree)
 {
@@ -385,7 +504,6 @@ static GPAOption *
 gpa_option_new_node_from_tree (xmlNodePtr tree, const guchar *id)
 {
 	GPAOption *option;
-	xmlChar *xmlval;
 	xmlNodePtr xmlc;
 	GSList *l;
 
@@ -493,6 +611,8 @@ static GPAOption *
 gpa_option_new_list_from_tree (xmlNodePtr tree, const guchar *id)
 {
 	GPAOption *option;
+	GPANode *globals;
+	GPANode *ref;
 	xmlChar *xmldef, *xmlval;
 	xmlNodePtr xmlc;
 	GSList *l;
@@ -509,36 +629,57 @@ gpa_option_new_list_from_tree (xmlNodePtr tree, const guchar *id)
 	has_default = FALSE;
 
 	for (xmlc = tree->xmlChildrenNode; xmlc != NULL; xmlc = xmlc->next) {
-		if (xmlc->type == XML_ELEMENT_NODE) {
-			if (!strcmp (xmlc->name, "Item")) {
-				GPANode *cho;
-				cho = gpa_option_new_from_tree (xmlc);
-				if (cho) {
-					l = g_slist_prepend (l, cho);
-					if (!strcmp (xmldef, cho->id)) has_default = TRUE;
-				}
-			} else if (!strcmp (xmlc->name, "Fill")) {
-				xmlval = xmlGetProp (xmlc, "Ref");
-				if (xmlval) {
-					GPANode *def, *ref;
-					def = gpa_defaults ();
-					ref = gpa_node_get_path_node (def, xmlval);
-					gpa_node_unref (def);
-					xmlFree (xmlval);
-					if (GPA_OPTION_IS_LIST (ref)) {
-						GPANode *lc;
-						for (lc = ((GPAOption *) ref)->children; lc != NULL; lc = lc->next) {
-							GPANode *new;
-							new = gpa_node_duplicate (lc);
-							l = g_slist_prepend (l, new);
-							if (!strcmp (xmldef, new->id)) has_default = TRUE;
-						}
-					}
-					if (ref) gpa_node_unref (ref);
-				}
-			} else {
-				g_warning ("Invalid list child in option tree %s", xmlc->name);
+		if (xmlc->type != XML_ELEMENT_NODE)
+			continue;
+		
+		if (!strcmp (xmlc->name, "Item")) {
+			GPANode *cho;
+			cho = gpa_option_new_from_tree (xmlc);
+			if (cho) {
+				l = g_slist_prepend (l, cho);
+				if (GPA_NODE_ID_COMPARE (cho, xmldef))
+					has_default = TRUE;
 			}
+			
+		} else if (!strcmp (xmlc->name, "Fill")) {
+			xmlval = xmlGetProp (xmlc, "Ref");
+			if (!xmlval)
+				continue;
+			
+			if (strncmp (xmlval, "Globals.", 8) != 0) {
+				g_warning ("You can only have refernences to Globals "
+					   "(at least for now) you tried referencing: %s\n", xmlval);
+				xmlFree (xmlval);
+				continue;
+			}
+
+			globals = GPA_NODE (gpa_root_get ());
+			if (!globals) {
+				g_warning ("Cannot read global configuration data");
+				continue;
+			}
+			ref = gpa_node_lookup (globals, xmlval + 8);
+			gpa_node_unref (globals);
+			if (!ref) {
+				g_warning ("Could not get hold of the reference pointed by %s\n",
+					   xmlval);
+				continue;
+			}
+			if (GPA_OPTION_IS_LIST (ref)) {
+				GPANode *lc;
+				for (lc = ((GPAOption *) ref)->children; lc != NULL; lc = lc->next) {
+					GPANode *new;
+					new = gpa_node_duplicate (lc);
+					l = g_slist_prepend (l, new);
+					if (GPA_NODE_ID_COMPARE (new, xmldef))
+						has_default = TRUE;
+				}
+			}
+			gpa_node_unref (ref);
+			xmlFree (xmlval);
+			xmlval = NULL;
+		} else {
+			g_warning ("Invalid list child in option tree %s", xmlc->name);
 		}
 	}
 
@@ -602,7 +743,8 @@ gpa_option_new_item_from_tree (xmlNodePtr tree, const guchar *id)
 			if (!strcmp (xmlc->name, "Option") || !strcmp (xmlc->name, "Key")) {
 				GPANode *cho;
 				cho = gpa_option_new_from_tree (xmlc);
-				if (cho) l = g_slist_prepend (l, cho);
+				if (cho)
+					l = g_slist_prepend (l, cho);
 			} else if (strcmp (xmlc->name, "Name")) {
 				g_warning ("Invalid tag in option tree %s", xmlc->name);
 			}
@@ -611,7 +753,7 @@ gpa_option_new_item_from_tree (xmlNodePtr tree, const guchar *id)
 
 	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
 	option->type = GPA_OPTION_TYPE_ITEM;
-	option->name = name;
+	option->name = gpa_node_attach (GPA_NODE (option), name);
 
 	while (l) {
 		GPANode *cho;
@@ -661,14 +803,19 @@ gpa_option_list_new_from_tree (xmlNodePtr tree)
 
 	l = NULL;
 	for (xmlc = tree->xmlChildrenNode; xmlc != NULL; xmlc = xmlc->next) {
-		if (!strcmp (xmlc->name, "Option") || !strcmp (xmlc->name, "Item") || !strcmp (xmlc->name, "Key")) {
+		if (!strcmp (xmlc->name, "Option") ||
+		    !strcmp (xmlc->name, "Item")   ||
+		    !strcmp (xmlc->name, "Key")) {
 			GPANode *option;
 			option = gpa_option_new_from_tree (xmlc);
-			if (option) l = g_slist_prepend (l, option);
+			if (option)
+				l = g_slist_prepend (l, option);
 		}
 	}
 
-	options = GPA_LIST (gpa_list_new (GPA_TYPE_OPTION, FALSE));
+	/* FIXME: I'm not sure if the default parameter should be false or not (Chema) */
+	options = gpa_list_new (GPA_TYPE_OPTION, "OptionsList", FALSE);
+	/* FIXME: I don't see a good reason to make options a list. (Chema) */
 
 	while (l) {
 		GPANode *option;
@@ -682,6 +829,20 @@ gpa_option_list_new_from_tree (xmlNodePtr tree)
 	return options;
 }
 
+/* Public interface */
+
+GPANode *
+gpa_option_create_key (GPAOption *option)
+{
+	g_return_val_if_fail (option != NULL, NULL);
+	g_return_val_if_fail (GPA_IS_OPTION (option), NULL);
+
+	if (GPA_OPTION_GET_CLASS (option)->create_key)
+		return GPA_OPTION_GET_CLASS (option)->create_key (option);
+
+	return NULL;
+}
+
 /* Strictly private helpers */
 
 GPAOption *
@@ -695,7 +856,7 @@ gpa_option_get_child_by_id (GPAOption *option, const guchar *id)
 	g_return_val_if_fail (*id != '\0', NULL);
 
 	for (child = option->children; child != NULL; child = child->next) {
-		if (child->id && !strcmp (id, child->id)) {
+		if (GPA_NODE_ID_COMPARE (child, id)) {
 			g_assert (GPA_IS_OPTION (child));
 			gpa_node_ref (child);
 			return GPA_OPTION (child);
@@ -782,7 +943,8 @@ gpa_option_key_new (const guchar *id, const guchar *value)
 
 	option = (GPAOption *) gpa_node_new (GPA_TYPE_OPTION, id);
 	option->type = GPA_OPTION_TYPE_KEY;
-	if (value) option->value = g_strdup (value);
+	if (value)
+		option->value = g_strdup (value);
 
 	return (GPANode *) option;
 }
@@ -808,7 +970,7 @@ gpa_option_node_append_child (GPAOptionNode *option, GPAOption *child)
 		ref->next = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
 	}
 
-	gpa_node_request_modified (GPA_NODE (option));
+	gpa_node_request_modified (GPA_NODE (option), 0);
 
 	return TRUE;
 }
@@ -838,7 +1000,7 @@ gpa_option_list_append_child (GPAOptionList *option, GPAOption *child)
 		option->value = g_strdup (GPA_NODE_ID (child));
 	}
 
-	gpa_node_request_modified (GPA_NODE (option));
+	gpa_node_request_modified (GPA_NODE (option), 0);
 
 	return TRUE;
 }
@@ -864,7 +1026,7 @@ gpa_option_item_append_child (GPAOptionItem *option, GPAOption *child)
 		ref->next = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
 	}
 
-	gpa_node_request_modified (GPA_NODE (option));
+	gpa_node_request_modified (GPA_NODE (option), 0);
 
 	return TRUE;
 }
@@ -890,7 +1052,7 @@ gpa_option_key_append_child (GPAOptionKey *option, GPAOption *child)
 		ref->next = gpa_node_attach_ref (GPA_NODE (option), GPA_NODE (child));
 	}
 
-	gpa_node_request_modified (GPA_NODE (option));
+	gpa_node_request_modified (GPA_NODE (option), 0);
 
 	return TRUE;
 }
@@ -900,10 +1062,8 @@ gpa_option_key_append_child (GPAOptionKey *option, GPAOption *child)
 static gboolean
 gpa_option_xml_check (xmlNodePtr node, gint def, gint val, gint name, gint children)
 {
-	xmlChar *xmlval;
-
 	if (GPA_UNAMBIGUOUS (def)) {
-		xmlval = xmlGetProp (node, "Default");
+		xmlChar *xmlval = xmlGetProp (node, "Default");
 		if (xmlval && GPA_FORBIDDEN (def)) {
 			g_warning ("Node does not have \"Default\" attribute");
 			xmlFree (xmlval);
@@ -913,10 +1073,11 @@ gpa_option_xml_check (xmlNodePtr node, gint def, gint val, gint name, gint child
 			xmlFree (xmlval);
 			return FALSE;
 		}
+		xmlFree (xmlval);
 	}
 
 	if (GPA_UNAMBIGUOUS (val)) {
-		xmlval = xmlGetProp (node, "Value");
+		xmlChar *xmlval = xmlGetProp (node, "Value");
 		if (xmlval && GPA_FORBIDDEN (val)) {
 			g_warning ("Node does not have \"Value\" attribute");
 			xmlFree (xmlval);
@@ -926,10 +1087,11 @@ gpa_option_xml_check (xmlNodePtr node, gint def, gint val, gint name, gint child
 			xmlFree (xmlval);
 			return FALSE;
 		}
+		xmlFree (xmlval);
 	}
 
 	if (GPA_UNAMBIGUOUS (name)) {
-		xmlval = gpa_xml_node_get_name (node);
+		xmlChar *xmlval = gpa_xml_node_get_name (node);
 		if (xmlval && GPA_FORBIDDEN (name)) {
 			g_warning ("Node does not have <Name> element");
 			xmlFree (xmlval);
@@ -939,6 +1101,7 @@ gpa_option_xml_check (xmlNodePtr node, gint def, gint val, gint name, gint child
 			xmlFree (xmlval);
 			return FALSE;
 		}
+		xmlFree (xmlval);
 	}
 
 	if (GPA_UNAMBIGUOUS (children)) {
