@@ -28,11 +28,13 @@
 
 #define __GP_TRANSPORT_FILE_C__
 
+#include "config.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <libgnomeprint/gnome-print.h>
+
 #include "gp-transport-file.h"
 
 static void gp_transport_file_class_init (GPTransportFileClass *klass);
@@ -101,7 +103,7 @@ gp_transport_file_finalize (GObject *object)
 
 	tf = GP_TRANSPORT_FILE (object);
 
-	if (tf->fd) {
+	if (tf->fd != -1) {
 		g_warning ("Destroying GPTransportFile with open file descriptor");
 	}
 
